#region Using Statements
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System;
#endregion

namespace FoodFight
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class FoodFight : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public static Player mainPlayer;

        private Level1 testLevel;
        private MainMenu menu;
        private PauseMenu pauseMenu;

        public static StateManager GameStateManager;

        public static float AspectRatio { get; set; }

        public static Rectangle SCREEN_SAFE;

        public static KeyBinding InputBindings;

        public FoodFight()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        private void InitializeGraphics()
        {
            graphics.PreferredBackBufferWidth = 1440;
            graphics.PreferredBackBufferHeight = 900;
            AspectRatio = graphics.GraphicsDevice.Viewport.AspectRatio;

            graphics.ApplyChanges();

            SCREEN_SAFE = graphics.GraphicsDevice.Viewport.TitleSafeArea;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            InitializeGraphics();

            GameStateManager = new StateManager(GameStates.MainMenu);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            pauseMenu = new PauseMenu(this.Content);
            InputBindings = new KeyBinding();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        private void ResetMouse(bool isVisible)
        {
            Mouse.SetPosition(SCREEN_SAFE.Center.X, SCREEN_SAFE.Center.Y);
            this.IsMouseVisible = isVisible;
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            switch (GameStateManager.Peek())
            {
                case GameStates.StartNewGame:
                    menu.Destroy();
                    GameStateManager.Push(GameStates.InGame);
                    break;
                case GameStates.MainMenu:
                    if (menu == null)
                    {
                        menu = new MainMenu(this.Content, GameStateManager);
                        ResetMouse(true);
                    }
                    menu.Update(gameTime);
                    break;
                case GameStates.InGame:
                    Mouse.SetPosition(SCREEN_SAFE.Center.X, SCREEN_SAFE.Center.Y);
                    if (testLevel == null)
                    {
                        testLevel = new Level1(this.Content);
                        ResetMouse(false);
                    }
                    testLevel.Update(gameTime);
                    mainPlayer.Update(gameTime);
                    mainPlayer.UpdateProjectiles(testLevel.Enemies, gameTime);
                    break;
                case GameStates.Paused:
                    pauseMenu.Update();
                    break;
                case GameStates.RequestQuit:
                    this.Exit();
                    break;
            };

            InputBindings.boardState = Keyboard.GetState();
            InputBindings.mouseState = Mouse.GetState();

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            if(GameStateManager.Contains(GameStates.MainMenu)){
                if (menu != null)
                {
                    spriteBatch.Begin();
                    menu.Draw(spriteBatch);
                    spriteBatch.End();
                }
            }
            
            if (GameStateManager.Contains(GameStates.InGame)) {
                if (testLevel != null)
                {
                    this.GraphicsDevice.RenderState.DepthBufferEnable = true;
                    this.GraphicsDevice.RenderState.DepthBufferWriteEnable = true;
                    testLevel.Render();
                    mainPlayer.Draw();

                    spriteBatch.Begin();
                    testLevel.Draw(spriteBatch);
                    spriteBatch.End();
                }
            }

            if (GameStateManager.Contains(GameStates.Paused)) {
                spriteBatch.Begin();
                pauseMenu.Draw(spriteBatch);
                spriteBatch.End();
            }

            base.Draw(gameTime);
        }
    }
}
