﻿#region Using Statements
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;
using System.Collections.Generic;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace FoodFight
{
    public class Player : Renderable
    {
        private PlayerCamera _cam;

        public int Score { get; set; }

        private List<Projectile> _bullets;
        private ContentManager _content;

        public PlayerCamera Camera
        {
            get { return _cam; }
        }

        public Player(ModelEntity playerModel, ContentManager content) : base(playerModel)
        {
            _model = playerModel;
            this.MovementSpeed = 20.0f;

            _cam = new PlayerCamera(this);

            _bullets = new List<Projectile>();
            _content = content;
        }

        public void Draw()
        {
            foreach (Projectile m in _bullets)
            {
                m.Draw(_cam);
            }

            this.Draw(_cam);
        }

        public override void Update(GameTime gameTime)
        {
            if (Keyboard.GetState().IsKeyDown(Keys.W))
            {
                Vector3 temp = this.Position;
                temp.Z -= MovementSpeed;
                this.Position = temp;

                _cam.MoveCamera(new Vector3(0.0f, 0.0f, -MovementSpeed));
            }

            if (Keyboard.GetState().IsKeyDown(Keys.S))
            {
                Vector3 temp = this.Position;
                temp.Z += MovementSpeed;
                this.Position = temp;

                _cam.MoveCamera(new Vector3(0.0f, 0.0f, MovementSpeed));
            }

            if (Keyboard.GetState().IsKeyDown(Keys.A))
            {
                Vector3 temp = this.Position;
                temp.X -= MovementSpeed;
                this.Position = temp;

                _cam.MoveCamera(new Vector3(-MovementSpeed, 0.0f, 0.0f));
            }

            if (Keyboard.GetState().IsKeyDown(Keys.D))
            {
                Vector3 temp = this.Position;
                temp.X += MovementSpeed;
                this.Position = temp;

                _cam.MoveCamera(new Vector3(MovementSpeed, 0.0f, 0.0f));
            }

            if (FoodFight.InputBindings.mouseState.LeftButton == ButtonState.Released
                && Mouse.GetState().LeftButton == ButtonState.Pressed)
            {
                ShootBullet();
            }

            _model.Update();
        }

        public void UpdateProjectiles(List<Enemy> enemies, GameTime gameTime)
        {
            List<Projectile> bulletsToRemove = new List<Projectile>();

            foreach (Projectile m in _bullets)
            {
                m.Update(gameTime);

                foreach (Enemy e in enemies)
                {
                    if (m.Model.CollisionSphere.Intersects(e.Model.CollisionSphere))
                    {
                        bulletsToRemove.Add(m);
                        e.Kill();
                    }
                    else if (m.Position.Z < this.Position.Z - 4000.0f)
                    {
                        bulletsToRemove.Add(m);
                    }
                }
            }

            foreach (Projectile m in bulletsToRemove)
            {
                _bullets.Remove(m);
            }
        }

        private void ShootBullet()
        {
            Projectile bullet = new Projectile(new ModelEntity(_content.Load<Model>("Models\\bullet"), Position - new Vector3(0.0f, 0.0f, 100.0f),
                new Vector3(0.2f, 0.2f, 0.2f), 50.0f));
            bullet.Rotation = new Vector3(MathHelper.ToRadians(90), 0.0f, 0.0f);

            _bullets.Add(bullet);
        }
    }
}
